@@ -17,7 +17,7 @@
 */
 /*
  *
- * Copyright (C) 2004 Mekensleep
+ * Copyright (C) 2004, 2006 Mekensleep
  *
  *	Mekensleep
  *	24 rue vieille du temple
@@ -41,13 +41,19 @@
  * Authors:
  *  Cedric PINSON <cpinson@freesheep.org>
  *  Loic Dachary <loic@gnu.org>
- *  Igor Kravtchenko <igor@ozos.net>
+ *  Igor Kravtchenko <igor@tsarevitch.org>
  *
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif // HAVE_CONFIG_H
+#endif
+
+#ifdef USE_NPROFILE
+#include <nprofile/profile.h>
+#else  // USE_NPROFILE
+#define NPROFILE_SAMPLE(a)
+#endif // USE_NPROFILE
 
 #include <osg/Notify>
 #include <osg/Texture2D>
@@ -66,7 +72,7 @@
 
 using namespace osgCal;
 
-static std::map<std::string, osg::ref_ptr<osg::Texture2D> > g_name2texture;
+extern std::map<std::string, osg::ref_ptr<osg::Texture2D> > g_filename2texture;
 
 class TextureInformation : public osg::Referenced
 {
@@ -118,33 +124,35 @@ CoreModel::~CoreModel()
 void CoreModel::garbageCollect(void) 
 {
   // remove textures if no other object references them
-	std::vector<int> texturesIds;
-	for(CoreMaterialId2Textures2D::iterator i = _coreMaterialId2Textures.begin(); i != _coreMaterialId2Textures.end(); i++)
-		texturesIds.push_back(i->first);
+  std::vector<int> texturesIds;
+  for(CoreMaterialId2Textures2D::iterator i = _coreMaterialId2Textures.begin(); i != _coreMaterialId2Textures.end(); i++)
+    texturesIds.push_back(i->first);
 
-	for (int i = 0; i < (int)texturesIds.size(); i++) {
-		CoreMaterialId2Textures2D::iterator t = _coreMaterialId2Textures.find(texturesIds[i]);
-		Textures2D& textures = t->second;
+  for (int i = 0; i < (int)texturesIds.size(); i++) {
+    CoreMaterialId2Textures2D::iterator t = _coreMaterialId2Textures.find(texturesIds[i]);
+    Textures2D& textures = t->second;
 
-		bool bErase = false;
-		if (textures.empty())
-			bErase = true;
-		else {
-			if (!textures[0].valid())
-				bErase = true;
-			else
-				bErase = textures[0]->referenceCount() < 2 ? true : false;
-		}
+    bool bErase = false;
+    if (textures.empty())
+      bErase = true;
+    else {
+      if (!textures[0].valid())
+        bErase = true;
+      else
+        bErase = textures[0]->referenceCount() < 2 ? true : false;
+    }
 
     if (bErase) {
-			int reference_count = textures[0]->referenceCount();
-      // sanity check
-      for (Textures2D::iterator j = textures.begin(); j != textures.end(); j++)
-				if ((*j)->referenceCount() != reference_count) {
-					CalError::setLastError(CalError::INVALID_HANDLE, __FILE__, __LINE__);
-					return;
-				}
-			_coreMaterialId2Textures.erase(t);
+      if(textures.size() > 0 && textures[0].valid()) {
+        int reference_count = textures[0]->referenceCount();
+        // sanity check
+        for (Textures2D::iterator j = textures.begin(); j != textures.end(); j++)
+          if ((*j)->referenceCount() != reference_count) {
+            CalError::setLastError(CalError::INVALID_HANDLE, __FILE__, __LINE__);
+            return;
+          }
+      }
+      _coreMaterialId2Textures.erase(t);
     }
   }
 }
@@ -178,7 +186,7 @@ CoreModel::Textures2D* CoreModel::getTextures2D(int coreMaterialId)
 
       std::string strFilename = pCoreMaterial->getMapFilename(mapId);
 
-			osg::Texture2D *texture = g_name2texture[strFilename].get();
+			osg::Texture2D *texture = g_filename2texture[strFilename].get();
 			if (!texture) {
 				osg::Image *img = osgDB::readImageFile(strFilename);
 				if(img == NULL) {
@@ -190,13 +198,17 @@ CoreModel::Textures2D* CoreModel::getTextures2D(int coreMaterialId)
  				texture = new osg::Texture2D();
 				texture->setImage(img);
 				texture->setUnRefImageDataAfterApply(true);
-				//texture->setInternalFormatMode(osg::Texture::USE_ARB_COMPRESSION);
+
+				texture->setFilter(osg::Texture2D::MIN_FILTER, osg::Texture2D::LINEAR_MIPMAP_NEAREST);
+				texture->setFilter(osg::Texture2D::MAG_FILTER, osg::Texture2D::LINEAR);
+				texture->setUseHardwareMipMapGeneration(false);
+
 				texture->setWrap(osg::Texture2D::WRAP_T, osg::Texture2D::REPEAT);
 				texture->setWrap(osg::Texture2D::WRAP_S, osg::Texture2D::REPEAT);
 
 				TextureInformation *ti = new TextureInformation(strFilename);
 				texture->setUserData(ti);
-				g_name2texture[strFilename] = texture;
+				g_filename2texture[strFilename] = texture;
 			}
 			textures.push_back(texture);
     }
@@ -206,6 +218,20 @@ CoreModel::Textures2D* CoreModel::getTextures2D(int coreMaterialId)
   return &_coreMaterialId2Textures[coreMaterialId];
 }
 
+bool CoreModel::getTexturesNameFromMaterialID(int _coreMaterialId, std::vector<std::string> &_res)
+{
+	CalCoreMaterial *pCoreMaterial = _calCoreModel.getCoreMaterial(_coreMaterialId);
+
+	if (pCoreMaterial == NULL)
+		return false;
+
+	int mapId;
+	for(mapId = 0; mapId < pCoreMaterial->getMapCount(); mapId++) {
+		std::string strFilename = pCoreMaterial->getMapFilename(mapId);
+		_res.push_back(strFilename);
+	}
+	return true;
+}
 
 bool CoreModel::SubUsingMeshId(int coreMeshId) 
 {
@@ -237,36 +263,3 @@ bool CoreModel::AddUsingMeshId(int coreMeshId)
   if(_notify > osg::NOTICE) osg::notify(osg::DEBUG_INFO) << "CoreModel::AddUsingMeshId: " << _mReferenceCountCoreMeshIds[coreMeshId] << " references on coreMeshId " << coreMeshId << std::endl;
   return true;
 }
-
-
-void CoreModel::addIncompatibility(const std::string &_mesh1, const std::string &_mesh2)
-{
-	std::vector<std::string> &vec1 = _incompatiblityEntries[_mesh1];
-	vec1.push_back(_mesh2);
-
-	std::vector<std::string> &vec2 = _incompatiblityEntries[_mesh2];
-	vec2.push_back(_mesh1);
-}
-
-bool CoreModel::isIncompatible(const std::string &_mesh1, const std::string &_mesh2)
-{
-	int i;
-
-	std::vector<std::string> &vec1 = _incompatiblityEntries[_mesh1];
-	int size1 = vec1.size();
-	for (i = 0; i < size1; i++) {
-		std::string &name = vec1[i];
-		if (name == _mesh2)
-			return true;
-	}
-
-	std::vector<std::string> &vec2 = _incompatiblityEntries[_mesh2];
-	int size2 = vec2.size();
-	for (i = 0; i < size2; i++) {
-		std::string &name = vec2[i];
-		if (name == _mesh1)
-			return true;
-	}
-
-	return false;
-}
