@@ -1,6 +1,6 @@
 /* -*- c++ -*-
 *
-* Copyright (C) 2004 Mekensleep
+* Copyright (C) 2004, 2006 Mekensleep
 *
 * Mekensleep
 * 24 rue vieille du temple
@@ -24,7 +24,7 @@
 * Authors:
 *  Loic Dachary <loic@gnu.org>
 *  Cedric Pinson <cpinson@freesheep.org>
-*  Igor Kravtchenko <igor@ozos.net>
+*  Igor Kravtchenko <igor@tsarevitch.org>
 *
 */
 
@@ -69,11 +69,21 @@
 // Author : Jerome Schmid <jerome.schmid@free.fr>
 
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #ifdef WIN32
 #include <windows.h>
 #include <mmsystem.h>
 #endif
 
+#ifdef USE_NPROFILE
+#include <nprofile/profile.h>
+#else  // USE_NPROFILE
+#define NPROFILE_SAMPLE(a)
+#endif // USE_NPROFILE
+
 #ifndef DATADIR
 #define DATADIR "."
 #endif // DATADIR
@@ -109,7 +119,6 @@
 #include <osgCal/SubMeshSoftware>
 #include "osgCal/CoreModel"
 
-
 static bool writeObjectXFG(const std::string& fileName, const osgCal::CoreModel& coreModel) 
 {
   return true;
@@ -159,6 +168,7 @@ public:
 
     osg::notify(osg::INFO)<<   "ReaderWriterXFG::readObject( "<<fileName.c_str()<<" )\n";
     osg::ref_ptr<osgCal::CoreModel> coreModel = new osgCal::CoreModel;
+		coreModel->setFilename(file);
 
     if(!readObjectXFG(fileName, *coreModel, options))
       return ReadResult("failed to load correctly " + fileName);
@@ -181,9 +191,23 @@ bool ReaderWriterXFG::readObjectXFG(const std::string& fileName, osgCal::CoreMod
   CalCoreModel* calCoreModel = coreModel.getCalCoreModel();
   osg::notify(osg::INFO)<< "Loading cal3d character from "<<fileName<< std::endl;
 
-  gchar* dirname = g_path_get_dirname(fileName.c_str());
-  const std::string dir = std::string(dirname);
-  g_free(dirname);
+	std::string dir;
+
+	if (!options) {
+		gchar *dirname	= g_path_get_dirname(fileName.c_str());
+	  dir = std::string(dirname);
+		g_free(dirname);
+	}
+	else {
+		const osgDB::FilePathList &fpl = options->getDatabasePathList();
+		if (fpl.size() > 0)
+			dir = options->getDatabasePathList().front();
+		else {
+			gchar *dirname	= g_path_get_dirname(fileName.c_str());
+		  dir = std::string(dirname);
+			g_free(dirname);
+		}
+	}
 
   coreModel._configurable=true;
 
@@ -322,6 +346,10 @@ bool ReaderWriterXFG::parseSpecifications(CalCoreModel* calCoreModel, osgCal::Co
                 ;
               else if(variable == std::string("material1"))
                 ;
+              else if(variable == std::string("collision"))
+                ;
+              else if(variable == std::string("software"))
+                ;
               else
                 osg::notify(osg::FATAL) << "unexpected attribute " << variable << "=\"" << value << "\" at " << xpath << " in " << path << std::endl;
               xmlFree((void*)value);
@@ -441,8 +469,6 @@ bool ReaderWriterXFG::parseSpecifications(CalCoreModel* calCoreModel, osgCal::Co
     xmlXPathFreeObject(xpathObj);
   }
 
-  //int animTime1 = timeGetTime();
-
   {
     char* xpath = "/cal3d/library/animation";
     xmlXPathObjectPtr xpathObj = xmlXPathEvalExpression((xmlChar*)xpath, xpathContext);
@@ -495,9 +521,26 @@ bool ReaderWriterXFG::parseSpecifications(CalCoreModel* calCoreModel, osgCal::Co
           osg::notify(osg::FATAL) << "name attribute not set or empty at occurence " << i << " of " << xpath << " in " << path << std::endl;
         osg::notify(osg::INFO) << "   animation " << filename<< std::endl;
 
+				//CalLoaderAnimationOptions opts;
+				//opts.bUseOneChunkForKeyframes = true;
+
+				//char str[200];
+				//sprintf(str, "Load Animation \"%s\"", filename.c_str());
+				NPROFILE_SAMPLE("Load Animation");
+
         int coreAnimationId = calCoreModel->loadCoreAnimation(filename, name);
+
+        //int coreAnimationId = calCoreModel->loadCoreAnimation(filename, name, &opts);
         if(coreAnimationId < 0)
           osg::notify(osg::FATAL) << "loadCoreAnimation(" << filename << "," << name << ") " << CalError::getLastErrorDescription() << " at " << xpath << " in " << path << std::endl;
+
+				CalCoreAnimation *anim = calCoreModel->getCoreAnimation(coreAnimationId);
+				if (anim) {
+
+					//CalSaverAnimationOptions opts;
+					//opts.bCompressKeyframes = true;
+					//CalSaver::saveCoreAnimation( anim->getFilename(), anim, &opts);
+				}
       }
     } else {
       osg::notify(osg::FATAL) << xpath << " not found in " << path << std::endl;
@@ -506,9 +549,13 @@ bool ReaderWriterXFG::parseSpecifications(CalCoreModel* calCoreModel, osgCal::Co
     xmlXPathFreeObject(xpathObj); 
   }
 
-//  int animTime2 = timeGetTime();
+  //int animTime2 = timeGetTime();
   //animTime2 -= animTime1;
 
+//	char str[200];
+	//sprintf(str, "%ld", animTime2);
+	//MessageBox(NULL, str, "hop", MB_OK); 
+
   {
     char* xpath = "/cal3d/common/mesh";
     xmlXPathObjectPtr xpathObj = xmlXPathEvalExpression((xmlChar*)xpath, xpathContext);
@@ -528,19 +575,22 @@ bool ReaderWriterXFG::parseSpecifications(CalCoreModel* calCoreModel, osgCal::Co
         switch(node->type) {
         case XML_ELEMENT_NODE:
           {
-            std::string line;
-            osgCal::CoreModel::MeshDescription map;
-            getNodeParameters(node,map);
-            line += "name" + std::string("=") + map["name"] + " ";
-            if(map.find("name") == map.end()) {
+            //std::string line;
+            //osgCal::CoreModel::MeshDescription map;
+						std::map<std::string, std::string> map;
+            getNodeParameters(node, map);
+						std::string meshName = map["name"];
+
+            //line += "name" + std::string("=") + map["name"] + " ";
+            if(meshName == "") {
               osg::notify(osg::FATAL) << "name attribute not set or empty " << xpath << " in " << path << std::endl;
               return false;
             }
-            if(duplicates.find(line) != duplicates.end()) {
-              osg::notify(osg::FATAL) << path << xpath << "duplicate mesh " << line << ", ignored" << std::endl;
+            if(duplicates.find(meshName) != duplicates.end()) {
+              osg::notify(osg::FATAL) << path << xpath << "duplicate mesh " << meshName << ", ignored" << std::endl;
             } else {
-              duplicates.insert(line);
-              coreModel._commonMeshes.push_back(map);
+              duplicates.insert(meshName);
+              coreModel._commonMeshes.push_back(meshName);
             }
           }
           break;
@@ -599,22 +649,35 @@ bool ReaderWriterXFG::parseSpecifications(CalCoreModel* calCoreModel, osgCal::Co
               std::map<std::string,std::string> smap;
               getNodeParameters(snode,smap);
               std::string nodeName=(const char*)snode->name;
-              if (smap.find("default")==smap.end()) {
-                osg::notify(osg::FATAL) << "error parameter \"default\" not found in parameter " << parameterName << " at " << xpath << std::endl;
-                return false;
-              }
-
-              if (smap.find("alterable")==smap.end()) {
-                osg::notify(osg::FATAL) << "error parameter \"alterable\" not found in parameter " << parameterName << " at " << xpath << std::endl;
-                return false;
-              }
 
               if (nodeName=="opacity") {
+								if (smap.find("default")==smap.end()) {
+									osg::notify(osg::FATAL) << "error parameter \"default\" not found in parameter " << parameterName << " at " << xpath << std::endl;
+									return false;
+								}
+
+								if (smap.find("alterable")==smap.end()) {
+									osg::notify(osg::FATAL) << "error parameter \"alterable\" not found in parameter " << parameterName << " at " << xpath << std::endl;
+									return false;
+								}
+
                 parameter._opacity._opacity=atoi(smap["default"].c_str());
                 parameter._opacity._modifiable=(smap["alterable"]=="true"?true:false);
                 if (smap.find("text")!=smap.end())
                   parameter._opacity._text=smap["text"];
+
               } else if (nodeName=="hue_set") {
+
+								if (smap.find("default")==smap.end()) {
+									osg::notify(osg::FATAL) << "error parameter \"default\" not found in parameter " << parameterName << " at " << xpath << std::endl;
+									return false;
+								}
+
+								if (smap.find("alterable")==smap.end()) {
+									osg::notify(osg::FATAL) << "error parameter \"alterable\" not found in parameter " << parameterName << " at " << xpath << std::endl;
+									return false;
+								}
+
                 std::string colorSelected=smap["default"];
                 parameter._colors._selected=atoi(colorSelected.c_str());
                 parameter._colors._modifiable=(smap["alterable"]=="true"?true:false);
@@ -667,6 +730,15 @@ bool ReaderWriterXFG::parseSpecifications(CalCoreModel* calCoreModel, osgCal::Co
                   return false;
                 }
               } else if (nodeName=="color_set") {
+								if (smap.find("default")==smap.end()) {
+									osg::notify(osg::FATAL) << "error parameter \"default\" not found in parameter " << parameterName << " at " << xpath << std::endl;
+									return false;
+								}
+
+								if (smap.find("alterable")==smap.end()) {
+									osg::notify(osg::FATAL) << "error parameter \"alterable\" not found in parameter " << parameterName << " at " << xpath << std::endl;
+									return false;
+								}
                 std::string colorSelected=smap["default"];
                 parameter._colors._selected=atoi(colorSelected.c_str());
                 parameter._colors._modifiable=(smap["alterable"]=="true"?true:false);
@@ -717,7 +789,8 @@ bool ReaderWriterXFG::parseSpecifications(CalCoreModel* calCoreModel, osgCal::Co
                   osg::notify(osg::FATAL) << "error color selected (" << parameterName << ") does not exist in the color set at " << xpath << std::endl;
                   return false;
                 }
-              } else {
+              } else if (nodeName=="showin") {
+							} else {
                 osg::notify(osg::FATAL) << "error unknow node type " << nodeName << " at " << xpath << std::endl;
                 assert(0);
               }
@@ -912,19 +985,21 @@ bool ReaderWriterXFG::parseSpecifications(CalCoreModel* calCoreModel, osgCal::Co
                           else {
 
                             duplicatesTextures.insert(targetName);
+
                             if (smap.find("mask") != smap.end()) {
                               if (smap["mask"] != "")
                                 slot._textures[targetName]._mask = dir + "/" + smap["mask"];
                               else
                                 slot._textures[targetName]._mask = "";
                             }
-                            /*
-                              slot._textures[targetName]._bAdditional = false;
-                              if (smap.find("additional") != smap.end()) {
-                              const std::string &additional = smap["additional"];
-                              slot._textures[targetName]._bAdditional = additional == "true" ? true : false;
-                              }
-                            */
+
+                            if (smap.find("alpha") != smap.end()) {
+                              if (smap["alpha"] != "")
+                                slot._textures[targetName]._alpha = dir + "/" + smap["alpha"];
+                              else
+                                slot._textures[targetName]._alpha = "";
+                            }
+
                             std::set<std::string> duplicatesLayer;
 
                             for (xmlNodePtr tnode = ssnode->children; tnode; tnode = tnode->next) {
@@ -1015,50 +1090,6 @@ bool ReaderWriterXFG::parseSpecifications(CalCoreModel* calCoreModel, osgCal::Co
     xmlXPathFreeObject(xpathObj); 
   }
 
-  {
-	  char *xpath = "/cal3d/incompatibility";
-	  xmlXPathObjectPtr xpathObj = xmlXPathEvalExpression((xmlChar*)xpath, xpathContext);
-	  if(xpathObj == NULL) {
-		  osg::notify(osg::FATAL) << "eval " << xpath << " file in " << path << std::endl;
-		  return false;
-	  }
-
-	  xmlNodeSetPtr nodes = xpathObj->nodesetval;
-
-	  for(int i = 0; i < nodes->nodeNr; i++) {
-		  xmlNodePtr node = nodes->nodeTab[i];
-
-		  switch(node->type) {
-			case XML_ELEMENT_NODE:
-				{
-
-					for (xmlNodePtr snode = node->children; snode; snode = snode->next) {
-						switch(snode->type) {
-			case XML_ELEMENT_NODE:
-				{
-					std::map<std::string, std::string> map;
-					getNodeParameters(snode, map);
-					const std::string &mesh_name1 = map["name1"];
-					const std::string &mesh_name2 = map["name2"];
-					coreModel.addIncompatibility(mesh_name1, mesh_name2);
-				}
-				break;
-
-			default:
-				break;
-						}
-					}
-				}
-				break;
-
-			default:
-				break;
-
-		  }
-	  }
-	  xmlXPathFreeObject(xpathObj); 
-  }
-
   return true;
 }
 
