@@ -1,6 +1,6 @@
 /*
  *
- * Copyright (C) 2004 Mekensleep
+ * Copyright (C) 2004, 2006 Mekensleep
  *
  *	Mekensleep
  *	24 rue vieille du temple
@@ -23,13 +23,19 @@
  *
  * Authors:
  *  Cedric PINSON <cpinson@freesheep.org>
- *  Igor Kravtchenko <igor@ozos.net>
+ *  Igor Kravtchenko <igor@tsarevitch.org>
  *
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif // HAVE_CONFIG_H
+#endif
+
+#ifdef USE_NPROFILE
+#include <nprofile/profile.h>
+#else  // USE_NPROFILE
+#define NPROFILE_SAMPLE(a)
+#endif // USE_NPROFILE
 
 #include <iostream>
 
@@ -37,15 +43,11 @@
 #include <osg/Texture2D>
 #include <osgCal/SubMeshHardware>
 #include <osgCal/Model>
-
-#ifdef USE_NPROFILE
-#include <nprofile/profile.h>
-#endif
+#include <osgCal/HardwareModel>
 
 using namespace osgCal;
 
 
-
 SubMeshHardware::SubMeshHardware() {
     std::cerr << "SubMeshHardware::SubMeshHardware(): You should never call this constructor!" << std::endl;
 }
@@ -56,7 +58,7 @@ SubMeshHardware::~SubMeshHardware()
 }
 
 
-SubMeshHardware::SubMeshHardware(CalModel *model, int mesh)
+SubMeshHardware::SubMeshHardware(Model *model, int mesh)
 {
   mModel=model;
   mMeshId=mesh;
@@ -67,28 +69,29 @@ SubMeshHardware::SubMeshHardware(CalModel *model, int mesh)
   setUseVertexBufferObjects(true);
 }
 
-void SubMeshHardware::InitHardwareMesh(CalIndex* idx,
+void SubMeshHardware::InitHardwareMesh(
                                 int nbIdxs,
                                 osg::VertexProgram* vp,
-                                CalHardwareModel* hardwareModel,
+                                GlueCalHardwareModel* hardwareModel,
                                 unsigned int* vbo,
-                                unsigned int indexInVBO)
+                                unsigned int indexInVBO,
+																int maxVertexIndex)
 {
   mHardwareModel = hardwareModel;
   mVp = vp;
   osg::StateSet *set = this->getOrCreateStateSet();
 
-  //mIarray = new osg::UIntArray(nbIdxs,(unsigned int*)idx);
-	if (sizeof(CalIndex) == sizeof(unsigned int) ) {
-		mIarray = new osg::UIntArray(nbIdxs, (unsigned int*)idx);
-	} else if (sizeof(CalIndex) == sizeof(unsigned short)) {
-		mIarray = new osg::UShortArray(nbIdxs, (unsigned short*)idx);
-	} else {
+	if (sizeof(CalIndex) == sizeof(unsigned int) )
+		indexInVBO = indexInVBO*sizeof(unsigned int);
+	else if (sizeof(CalIndex) == sizeof(unsigned short))
+		indexInVBO = indexInVBO*sizeof(unsigned short);
+	else
 		assert(0 && "Invalid type for index");
-	}
 
   _vbo = vbo;
   _indexVbo = indexInVBO;
+	_nbIdxs = nbIdxs;
+	_maxVertexIndex = maxVertexIndex;
 
   set->setAttributeAndModes( mVp, osg::StateAttribute::ON /*| osg::StateAttribute::PROTECTED*/); // | osg::StateAttribute::OVERRIDE /* | osg::StateAttribute::PROTECTED*/);
 }
@@ -100,6 +103,7 @@ void SubMeshHardware::drawImplementation(osg::State& state) const
     NPROFILE_SAMPLE("SubMeshHardware::drawImplementation");
 #endif
 
+//		return;
 //   std::cout << "min " << _bbox._min << " max " << _bbox._max << std::endl;
 
   // no texture we don't draw mesh (for collision mesh)
@@ -120,18 +124,20 @@ void SubMeshHardware::drawImplementation(osg::State& state) const
   const Extensions* extensions = getExtensions(state.getContextID(),true);
 
   extensions->glBindBuffer(GL_ARRAY_BUFFER_ARB, _vbo[MODEL_VBO_GEOMETRY]);
-  state.setVertexPointer(3, GL_FLOAT, 64, 0);
-  state.setNormalPointer(GL_FLOAT, 64, (GLvoid*)12);
-  state.setTexCoordPointer(0, 2, GL_FLOAT, 64, (GLvoid*)24);
-	state.setVertexAttribPointer(1, 4 , GL_FLOAT, false, 64, (GLvoid*)32);
-  state.setVertexAttribPointer(6, 4 , GL_FLOAT, false, 64, (GLvoid*)48);
+  state.setNormalPointer(GL_FLOAT, 88, (GLvoid*)12);
+  state.setTexCoordPointer(0, 2, GL_FLOAT, 88, (GLvoid*)24);
+  state.setTexCoordPointer(1, 2, GL_FLOAT, 88, (GLvoid*)32);
+	state.setVertexAttribPointer(1, 4 , GL_FLOAT, false, 88, (GLvoid*)40);
+  state.setVertexAttribPointer(6, 4 , GL_FLOAT, false, 88, (GLvoid*)56);
+	state.setColorPointer(4, GL_FLOAT, 88, (GLvoid*) 72);
+  state.setVertexPointer(3, GL_FLOAT, 88, 0);
 
   osg::VertexProgram::Extensions* vpExt = osg::VertexProgram::getExtensions(state.getContextID(),true);
 
   int boneId;
   for(boneId = 0; boneId < mHardwareModel->getBoneCount(); boneId++) {
-    CalQuaternion rotationBoneSpace = mHardwareModel->getRotationBoneSpace(boneId, mModel->getSkeleton());
-    CalVector translationBoneSpace = mHardwareModel->getTranslationBoneSpace(boneId, mModel->getSkeleton());
+    CalQuaternion rotationBoneSpace = mHardwareModel->getRotationBoneSpace(boneId, mModel->getCalModel()->getSkeleton());
+    CalVector translationBoneSpace = mHardwareModel->getTranslationBoneSpace(boneId, mModel->getCalModel()->getSkeleton());
 
     CalMatrix rotationMatrix = rotationBoneSpace;
     float transformation[12];
@@ -155,20 +161,20 @@ void SubMeshHardware::drawImplementation(osg::State& state) const
   }
 
   extensions->glBindBuffer(GL_ELEMENT_ARRAY_BUFFER_ARB, _vbo[MODEL_VBO_INDEX]);
-  unsigned int idxP = 0 + _indexVbo;
 
 	if ( sizeof(CalIndex) == sizeof(unsigned int)) {
-		osg::UIntArray* array = (osg::UIntArray*) mIarray.get();
-		glDrawElements(GL_TRIANGLES, 
-									 array->size(),
-									 GL_UNSIGNED_INT,
-									 (GLvoid*)idxP); 
-	} else if (sizeof(CalIndex) == sizeof(unsigned short)) {
-		osg::UShortArray* array= (osg::UShortArray*) mIarray.get();
-		glDrawElements(GL_TRIANGLES, 
-									 array->size(),
-									 GL_UNSIGNED_SHORT,
-									 (GLvoid*)idxP); 
+#ifndef OSG_GL_DRAWRANGELEMENTS
+		glDrawElements(GL_TRIANGLES, _nbIdxs, GL_UNSIGNED_INT, (GLvoid*)_indexVbo);
+#else
+		extensions->glDrawRangeElements(GL_TRIANGLES, 0, _maxVertexIndex, _nbIdxs, GL_UNSIGNED_INT, (GLvoid*) _indexVbo);
+#endif
+	}
+	else if (sizeof(CalIndex) == sizeof(unsigned short)) {
+#ifndef OSG_GL_DRAWRANGELEMENTS
+		glDrawElements(GL_TRIANGLES, _nbIdxs, GL_UNSIGNED_SHORT, (GLvoid*)_indexVbo);
+#else
+		extensions->glDrawRangeElements(GL_TRIANGLES, 0, _maxVertexIndex, _nbIdxs, GL_UNSIGNED_SHORT, (GLvoid*) _indexVbo);
+#endif
 	} else {
 		assert(0 && "invalid index size");
 	}
@@ -188,19 +194,19 @@ osg::Object* SubMeshHardware::clone(const osg::CopyOp&) const
 
 
 
-#if OSG_VERSION_RELEASE != 9
+#if OSG_VERSION_RELEASE != 9 && OSG_VERSION_MAJOR != 1
 bool SubMeshHardware::computeBound() const
 {
   _bbox=_staticbbox;
 //   _bbox.init();
-  _bbox_computed=true; // Force calculation on each frame
+	_bbox_computed = false;
   return true;
 }
-#else // OSG_VERSION_RELEASE != 9
+#else // OSG_VERSION_RELEASE != 9 && OSG_VERSION_MAJOR != 1
 osg::BoundingBox SubMeshHardware::computeBound() const
 {
   _boundingBox = _staticbbox;
-  _boundingBoxComputed = true;
+	_boundingBoxComputed = false;
   return _boundingBox;
 }
-#endif // OSG_VERSION_RELEASE != 9
+#endif // OSG_VERSION_RELEASE != 9 && OSG_VERSION_MAJOR != 1
