@@ -1,6 +1,6 @@
 /*
  *
- * Copyright (C) 2004, 2005 Mekensleep
+ * Copyright (C) 2004, 2005, 2006 Mekensleep
  *
  *  Mekensleep
  *  24 rue vieille du temple
@@ -29,16 +29,18 @@
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif // HAVE_CONFIG_H
+#endif
+
+#ifdef USE_NPROFILE
+#include <nprofile/profile.h>
+#else  // USE_NPROFILE
+#define NPROFILE_SAMPLE(a)
+#endif // USE_NPROFILE
 
 #ifdef WIN32
 #  include <windows.h>
 #endif // WIN32
 
-#ifdef USE_NPROFILE
-#include <nprofile/profile.h>
-#endif
-
 #include <cal3d/cal3d.h>
 
 #include <osg/Notify>
@@ -112,7 +114,18 @@ bool SubMeshSoftware::update(void)
   {
     pNormalBuffer = (float*)(dynamic_cast<osg::Vec3Array*>(getNormalArray())->getDataPointer());
   }
+/*
+	char *pColorBuffer = NULL;
+  if(!_invisible)
+  {
 
+#if OSG_VERSION_MAJOR != 1
+    pColorBuffer = (char*)(dynamic_cast<osg::UByte4Array*>(getColorArray())->getDataPointer());
+#else // OSG_VERSION_MAJOR != 1
+    pColorBuffer = (char*)(dynamic_cast<osg::Vec4ubArray*>(getColorArray())->getDataPointer());
+#endif // OSG_VERSION_MAJOR != 1
+  }
+*/
   memset(pVertexBuffer, 0, calCoreSubmesh->getVertexCount() * 3 * sizeof(float));
   if (NULL != pNormalBuffer)
   {
@@ -160,13 +173,19 @@ bool SubMeshSoftware::update(void)
       }
     }
 
-    // save vertex position
+		// save vertex position
     pVertexBuffer += 3;
 
     if (NULL != pNormalBuffer)
     {
       pNormalBuffer += 3;
     }
+/*
+		if (NULL != pColorBuffer)
+		{
+			pColorBuffer += 4;
+		}
+		*/
   }
 
   dirtyBound();
@@ -179,33 +198,35 @@ void SubMeshSoftware::drawImplementation(osg::State& state) const
 #ifdef USE_NPROFILE
   NPROFILE_SAMPLE("SubMeshSoftware::drawImplementation");
 #endif
+
   if(!_invisible)
     Geometry::drawImplementation(state);
 }
 
-#if OSG_VERSION_RELEASE != 9
+#if OSG_VERSION_RELEASE != 9 && OSG_VERSION_MAJOR != 1
 bool SubMeshSoftware::computeBound() const
 {
   if(_supportsPrimitiveFunctor) {
-    return Geometry::computeBound();
+    bool res = Geometry::computeBound();
+		return res;
   } else {
     _bbox=_staticbbox;
-    _bbox_computed = true;
+    _bbox_computed = false;
     return true;
   }
 }
-#else // OSG_VERSION_RELEASE != 9
+#else // OSG_VERSION_RELEASE != 9 && OSG_VERSION_MAJOR != 1
 osg::BoundingBox SubMeshSoftware::computeBound() const
 {
   if(_supportsPrimitiveFunctor) {
     return Geometry::computeBound();
   } else {
     _boundingBox = _staticbbox;
-    _boundingBoxComputed = true;
+    _boundingBoxComputed = false;
     return _boundingBox;
   }
 }
-#endif // OSG_VERSION_RELEASE != 9
+#endif // OSG_VERSION_RELEASE != 9 && OSG_VERSION_MAJOR != 1
 
 static int coreMeshId2MeshId(CalModel* calModel, int coreMeshId) {
   CalMesh* mesh = calModel->getMesh(coreMeshId);
@@ -267,7 +288,19 @@ bool SubMeshSoftware::create(void)
       setTexCoordArray(mapId, array);
     }
   }
-
+/*
+	// vertex color
+  if(!_invisible) {
+    int count = calRenderer->getVertexCount();
+#if OSG_VERSION_MAJOR != 1
+    osg::UByte4Array *array = new osg::UByte4Array(count);
+#else // OSG_VERSION_MAJOR != 1
+    osg::Vec4ubArray *array = new osg::Vec4ubArray(count);
+#endif // OSG_VERSION_MAJOR != 1
+    setColorArray(array);
+    setColorBinding(osg::Geometry::BIND_PER_VERTEX);
+	}
+*/
   // faces
   {
     int count = calRenderer->getFaceCount();
